@@ -366,3 +366,52 @@ func TestGrpcRegistryChecker(t *testing.T) {
 		})
 	}
 }
+
+func TestGetPodImageID(t *testing.T) {
+	var table = []struct {
+		description string
+		pod         *corev1.Pod
+		result      string
+	}{
+		{
+			description: "no pod status: return nothing",
+			pod:         &corev1.Pod{Status: corev1.PodStatus{ContainerStatuses: []corev1.ContainerStatus{}}},
+			result:      "",
+		},
+		{
+			description: "pod has status: return status",
+			pod:         &corev1.Pod{Status: corev1.PodStatus{ContainerStatuses: []corev1.ContainerStatus{{ImageID: "xyz123"}}}},
+			result:      "xyz123",
+		},
+	}
+
+	for i, tt := range table {
+		require.Equal(t, tt.result, getPodImageID(tt.pod), table[i].description)
+	}
+}
+
+func TestUpdatePodByDigest(t *testing.T) {
+	var table = []struct {
+		description string
+		updatePod   *corev1.Pod
+		servingPods []*corev1.Pod
+		result      bool
+	}{
+		{
+			description: "pod image ids match: not update from the registry: return false",
+			updatePod:   &corev1.Pod{Status: corev1.PodStatus{ContainerStatuses: []corev1.ContainerStatus{{ImageID: "xyz123"}}}},
+			servingPods: []*corev1.Pod{{Status: corev1.PodStatus{ContainerStatuses: []corev1.ContainerStatus{{ImageID: "xyz123"}}}}},
+			result:      false,
+		},
+		{
+			description: "pod image ids do not match:  updated image on the registry: return true",
+			updatePod:   &corev1.Pod{Status: corev1.PodStatus{ContainerStatuses: []corev1.ContainerStatus{{ImageID: "abc456"}}}},
+			servingPods: []*corev1.Pod{{Status: corev1.PodStatus{ContainerStatuses: []corev1.ContainerStatus{{ImageID: "xyz123"}}}}},
+			result:      true,
+		},
+	}
+
+	for i, tt := range table {
+		require.Equal(t, tt.result, updatePodByDigest(tt.updatePod, tt.servingPods), table[i].description)
+	}
+}
